'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _has2 = require('lodash/has');

var _has3 = _interopRequireDefault(_has2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactAce = require('react-ace');

var _reactAce2 = _interopRequireDefault(_reactAce);

var _patchEvent = require('part:@sanity/form-builder/patch-event');

var _patchEvent2 = _interopRequireDefault(_patchEvent);

var _default = require('part:@sanity/components/formfields/default');

var _default2 = _interopRequireDefault(_default);

var _default3 = require('part:@sanity/components/fieldsets/default');

var _default4 = _interopRequireDefault(_default3);

var _default5 = require('part:@sanity/components/selects/default');

var _default6 = _interopRequireDefault(_default5);

var _Fieldset = require('./Fieldset.css');

var _Fieldset2 = _interopRequireDefault(_Fieldset);

var _createHighlightMarkers = require('./createHighlightMarkers');

var _createHighlightMarkers2 = _interopRequireDefault(_createHighlightMarkers);

var _config = require('./config');

require('brace/mode/batchfile');

require('brace/mode/css');

require('brace/mode/html');

require('brace/mode/javascript');

require('brace/mode/json');

require('brace/mode/jsx');

require('brace/mode/markdown');

require('brace/mode/php');

require('brace/mode/sh');

require('brace/mode/text');

require('brace/theme/github');

require('brace/theme/monokai');

require('brace/theme/terminal');

require('brace/theme/tomorrow');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-enable import/no-unassigned-import */

/* eslint-disable import/no-unassigned-import */
function compareNumbers(numA, numB) {
  return numA - numB;
}

class CodeInput extends _react.PureComponent {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.state = {
      hasFocus: false
    }, this.handleCodeChange = code => {
      var _props = this.props;
      const type = _props.type,
            onChange = _props.onChange;

      const path = ['code'];
      const fixedLanguage = (0, _get3.default)(type, 'options.language');

      onChange(_patchEvent2.default.from([(0, _patchEvent.setIfMissing)({ _type: type.name, language: fixedLanguage }), code ? (0, _patchEvent.set)(code, path) : (0, _patchEvent.unset)(path)]));
    }, this.handleToggleSelectLine = lineNumber => {
      var _props2 = this.props;
      const type = _props2.type,
            onChange = _props2.onChange,
            value = _props2.value;

      const path = ['highlightedLines'];
      const highlightedLines = value && value.highlightedLines || [];

      let position = highlightedLines.indexOf(lineNumber);
      const patches = [(0, _patchEvent.setIfMissing)({ _type: type.name }), (0, _patchEvent.setIfMissing)([], ['highlightedLines'])];
      const addLine = position === -1;

      if (addLine) {
        // New element, figure out where to add it so it sorts correctly
        const sorted = highlightedLines.concat(lineNumber).sort(compareNumbers);
        position = sorted.indexOf(lineNumber);
        patches.push((0, _patchEvent.insert)([lineNumber], 'before', path.concat(position === sorted.length - 1 ? -1 : position)));
      } else if (highlightedLines.length === 1) {
        // Last element removed, unset whole path
        patches.push((0, _patchEvent.unset)(path));

        // Temporary workaround for bug in react-ace
        // (https://github.com/securingsincity/react-ace/issues/229)
        const editor = this.editor

        // Remove all markers from editor
        ;[true, false].forEach(inFront => {
          const currentMarkers = editor.getSession().getMarkers(inFront);
          Object.keys(currentMarkers).forEach(marker => {
            editor.getSession().removeMarker(currentMarkers[marker].id);
          });
        });
      } else {
        // Removed, but not the last element, remove single item
        patches.push((0, _patchEvent.unset)(path.concat(position)));
      }

      onChange(_patchEvent2.default.from(patches));
    }, this.handleGutterMouseDown = event => {
      const target = event.domEvent.target;
      if (target.classList.contains('ace_gutter-cell')) {
        const row = event.getDocumentPosition().row;
        this.handleToggleSelectLine(row + 1); // Ace starts at row 0
      }
    }, this.handleEditorLoad = editor => {
      this.editor = editor;
      this.editor.focus();
      this.editor.on('guttermousedown', this.handleGutterMouseDown);
    }, this.handleLanguageChange = item => {
      var _props3 = this.props;
      const type = _props3.type,
            onChange = _props3.onChange;

      const path = ['language'];
      onChange(_patchEvent2.default.from([(0, _patchEvent.setIfMissing)({ _type: type.name }), item ? (0, _patchEvent.set)(item.value, path) : (0, _patchEvent.unset)(path)]));
    }, this.renderEditor = () => {
      var _props4 = this.props;
      const value = _props4.value,
            type = _props4.type;

      const fixedLanguage = (0, _get3.default)(type, 'options.language');
      return _react2.default.createElement(_reactAce2.default, {
        mode: value && value.language || fixedLanguage || 'text',
        theme: this.getTheme(),
        width: '100%',
        onChange: this.handleCodeChange,
        name: `${this._inputId}__aceEditor`,
        value: value && value.code || '',
        markers: value && value.highlightedLines ? (0, _createHighlightMarkers2.default)(value.highlightedLines) : null,
        onLoad: this.handleEditorLoad,
        tabSize: 2,
        setOptions: _config.ACE_SET_OPTIONS,
        editorProps: _config.ACE_EDITOR_PROPS
      });
    }, _temp;
  }

  componentWillUnmount() {
    this.editor.removeListener('guttermousedown', this.handleGutterMouseDown);
  }

  getLanguageAlternatives() {
    return (0, _get3.default)(this.props.type, 'options.languageAlternatives') || _config.SUPPORTED_LANGUAGES;
  }

  getTheme() {
    const preferredTheme = (0, _get3.default)(this.props.type, 'options.theme');
    return preferredTheme && _config.SUPPORTED_THEMES.find(theme => theme === preferredTheme) ? preferredTheme : _config.DEFAULT_THEME;
  }

  render() {
    var _props5 = this.props;
    const value = _props5.value,
          type = _props5.type,
          level = _props5.level;

    const languages = this.getLanguageAlternatives().slice();

    if ((0, _has3.default)(type, 'options.language')) {
      return _react2.default.createElement(
        _default4.default,
        { styles: _Fieldset2.default, legend: type.title, description: type.description },
        this.renderEditor()
      );
    }

    const selectedLanguage = value && value.language ? this.getLanguageAlternatives().find(item => item.value === value.language) : undefined;

    if (!selectedLanguage) {
      languages.unshift({ title: 'Select language' });
    }
    const languageField = type.fields.find(field => field.name === 'language');

    return _react2.default.createElement(
      _default4.default,
      { legend: type.title, description: type.description },
      _react2.default.createElement(
        _default2.default,
        { level: level + 1, label: languageField.type.title },
        _react2.default.createElement(_default6.default, {
          onChange: this.handleLanguageChange,
          value: selectedLanguage,
          items: languages
        })
      ),
      _react2.default.createElement(
        _default2.default,
        { label: type.title, level: level + 1 },
        this.renderEditor()
      )
    );
  }
}
exports.default = CodeInput;
CodeInput.propTypes = {
  level: _propTypes2.default.number.isRequired,
  value: _propTypes2.default.shape({
    _type: _propTypes2.default.string,
    code: _propTypes2.default.string,
    language: _propTypes2.default.string,
    highlightedLines: _propTypes2.default.array
  }),
  type: _propTypes2.default.shape({
    name: _propTypes2.default.string,
    title: _propTypes2.default.string,
    description: _propTypes2.default.string,
    fields: _propTypes2.default.arrayOf(_propTypes2.default.shape({
      name: _propTypes2.default.string.isRequired
    }))
  }).isRequired,
  onChange: _propTypes2.default.func
};
CodeInput.defaultProps = {
  onChange() {}
};